"""
Клиентская библиотека для работы с LightRAG API.
"""

from lightrag_mcp.client.light_rag_server_api_client import AuthenticatedClient, Client, models
from lightrag_mcp.client.light_rag_server_api_client.api import default, documents, query

__all__ = ["Client", "AuthenticatedClient", "documents", "query", "default", "models"]



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.query_request_mode import QueryRequestMode
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.query_request_conversation_history_type_0_item import QueryRequestConversationHistoryType0Item


T = TypeVar("T", bound="QueryRequest")


@_attrs_define
class QueryRequest:
    """
    Attributes:
        query (str): The query text
        mode (Union[Unset, QueryRequestMode]): Query mode Default: QueryRequestMode.HYBRID.
        only_need_context (Union[None, Unset, bool]): If True, only returns the retrieved context without generating a
            response.
        only_need_prompt (Union[None, Unset, bool]): If True, only returns the generated prompt without producing a
            response.
        response_type (Union[None, Unset, str]): Defines the response format. Examples: 'Multiple Paragraphs', 'Single
            Paragraph', 'Bullet Points'.
        top_k (Union[None, Unset, int]): Number of top items to retrieve. Represents entities in 'local' mode and
            relationships in 'global' mode.
        max_token_for_text_unit (Union[None, Unset, int]): Maximum number of tokens allowed for each retrieved text
            chunk.
        max_token_for_global_context (Union[None, Unset, int]): Maximum number of tokens allocated for relationship
            descriptions in global retrieval.
        max_token_for_local_context (Union[None, Unset, int]): Maximum number of tokens allocated for entity
            descriptions in local retrieval.
        hl_keywords (Union[None, Unset, list[str]]): List of high-level keywords to prioritize in retrieval.
        ll_keywords (Union[None, Unset, list[str]]): List of low-level keywords to refine retrieval focus.
        conversation_history (Union[None, Unset, list['QueryRequestConversationHistoryType0Item']]): Stores past
            conversation history to maintain context. Format: [{'role': 'user/assistant', 'content': 'message'}].
        history_turns (Union[None, Unset, int]): Number of complete conversation turns (user-assistant pairs) to
            consider in the response context.
    """

    query: str
    mode: Union[Unset, QueryRequestMode] = QueryRequestMode.HYBRID
    only_need_context: Union[None, Unset, bool] = UNSET
    only_need_prompt: Union[None, Unset, bool] = UNSET
    response_type: Union[None, Unset, str] = UNSET
    top_k: Union[None, Unset, int] = UNSET
    max_token_for_text_unit: Union[None, Unset, int] = UNSET
    max_token_for_global_context: Union[None, Unset, int] = UNSET
    max_token_for_local_context: Union[None, Unset, int] = UNSET
    hl_keywords: Union[None, Unset, list[str]] = UNSET
    ll_keywords: Union[None, Unset, list[str]] = UNSET
    conversation_history: Union[None, Unset, list["QueryRequestConversationHistoryType0Item"]] = UNSET
    history_turns: Union[None, Unset, int] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        query = self.query

        mode: Union[Unset, str] = UNSET
        if not isinstance(self.mode, Unset):
            mode = self.mode.value

        only_need_context: Union[None, Unset, bool]
        if isinstance(self.only_need_context, Unset):
            only_need_context = UNSET
        else:
            only_need_context = self.only_need_context

        only_need_prompt: Union[None, Unset, bool]
        if isinstance(self.only_need_prompt, Unset):
            only_need_prompt = UNSET
        else:
            only_need_prompt = self.only_need_prompt

        response_type: Union[None, Unset, str]
        if isinstance(self.response_type, Unset):
            response_type = UNSET
        else:
            response_type = self.response_type

        top_k: Union[None, Unset, int]
        if isinstance(self.top_k, Unset):
            top_k = UNSET
        else:
            top_k = self.top_k

        max_token_for_text_unit: Union[None, Unset, int]
        if isinstance(self.max_token_for_text_unit, Unset):
            max_token_for_text_unit = UNSET
        else:
            max_token_for_text_unit = self.max_token_for_text_unit

        max_token_for_global_context: Union[None, Unset, int]
        if isinstance(self.max_token_for_global_context, Unset):
            max_token_for_global_context = UNSET
        else:
            max_token_for_global_context = self.max_token_for_global_context

        max_token_for_local_context: Union[None, Unset, int]
        if isinstance(self.max_token_for_local_context, Unset):
            max_token_for_local_context = UNSET
        else:
            max_token_for_local_context = self.max_token_for_local_context

        hl_keywords: Union[None, Unset, list[str]]
        if isinstance(self.hl_keywords, Unset):
            hl_keywords = UNSET
        elif isinstance(self.hl_keywords, list):
            hl_keywords = self.hl_keywords

        else:
            hl_keywords = self.hl_keywords

        ll_keywords: Union[None, Unset, list[str]]
        if isinstance(self.ll_keywords, Unset):
            ll_keywords = UNSET
        elif isinstance(self.ll_keywords, list):
            ll_keywords = self.ll_keywords

        else:
            ll_keywords = self.ll_keywords

        conversation_history: Union[None, Unset, list[dict[str, Any]]]
        if isinstance(self.conversation_history, Unset):
            conversation_history = UNSET
        elif isinstance(self.conversation_history, list):
            conversation_history = []
            for conversation_history_type_0_item_data in self.conversation_history:
                conversation_history_type_0_item = conversation_history_type_0_item_data.to_dict()
                conversation_history.append(conversation_history_type_0_item)

        else:
            conversation_history = self.conversation_history

        history_turns: Union[None, Unset, int]
        if isinstance(self.history_turns, Unset):
            history_turns = UNSET
        else:
            history_turns = self.history_turns

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "query": query,
            }
        )
        if mode is not UNSET:
            field_dict["mode"] = mode
        if only_need_context is not UNSET:
            field_dict["only_need_context"] = only_need_context
        if only_need_prompt is not UNSET:
            field_dict["only_need_prompt"] = only_need_prompt
        if response_type is not UNSET:
            field_dict["response_type"] = response_type
        if top_k is not UNSET:
            field_dict["top_k"] = top_k
        if max_token_for_text_unit is not UNSET:
            field_dict["max_token_for_text_unit"] = max_token_for_text_unit
        if max_token_for_global_context is not UNSET:
            field_dict["max_token_for_global_context"] = max_token_for_global_context
        if max_token_for_local_context is not UNSET:
            field_dict["max_token_for_local_context"] = max_token_for_local_context
        if hl_keywords is not UNSET:
            field_dict["hl_keywords"] = hl_keywords
        if ll_keywords is not UNSET:
            field_dict["ll_keywords"] = ll_keywords
        if conversation_history is not UNSET:
            field_dict["conversation_history"] = conversation_history
        if history_turns is not UNSET:
            field_dict["history_turns"] = history_turns

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.query_request_conversation_history_type_0_item import QueryRequestConversationHistoryType0Item

        d = dict(src_dict)
        query = d.pop("query")

        _mode = d.pop("mode", UNSET)
        mode: Union[Unset, QueryRequestMode]
        if isinstance(_mode, Unset):
            mode = UNSET
        else:
            mode = QueryRequestMode(_mode)

        def _parse_only_need_context(data: object) -> Union[None, Unset, bool]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, bool], data)

        only_need_context = _parse_only_need_context(d.pop("only_need_context", UNSET))

        def _parse_only_need_prompt(data: object) -> Union[None, Unset, bool]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, bool], data)

        only_need_prompt = _parse_only_need_prompt(d.pop("only_need_prompt", UNSET))

        def _parse_response_type(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        response_type = _parse_response_type(d.pop("response_type", UNSET))

        def _parse_top_k(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        top_k = _parse_top_k(d.pop("top_k", UNSET))

        def _parse_max_token_for_text_unit(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        max_token_for_text_unit = _parse_max_token_for_text_unit(d.pop("max_token_for_text_unit", UNSET))

        def _parse_max_token_for_global_context(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        max_token_for_global_context = _parse_max_token_for_global_context(d.pop("max_token_for_global_context", UNSET))

        def _parse_max_token_for_local_context(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        max_token_for_local_context = _parse_max_token_for_local_context(d.pop("max_token_for_local_context", UNSET))

        def _parse_hl_keywords(data: object) -> Union[None, Unset, list[str]]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                hl_keywords_type_0 = cast(list[str], data)

                return hl_keywords_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, list[str]], data)

        hl_keywords = _parse_hl_keywords(d.pop("hl_keywords", UNSET))

        def _parse_ll_keywords(data: object) -> Union[None, Unset, list[str]]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                ll_keywords_type_0 = cast(list[str], data)

                return ll_keywords_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, list[str]], data)

        ll_keywords = _parse_ll_keywords(d.pop("ll_keywords", UNSET))

        def _parse_conversation_history(
            data: object,
        ) -> Union[None, Unset, list["QueryRequestConversationHistoryType0Item"]]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                conversation_history_type_0 = []
                _conversation_history_type_0 = data
                for conversation_history_type_0_item_data in _conversation_history_type_0:
                    conversation_history_type_0_item = QueryRequestConversationHistoryType0Item.from_dict(
                        conversation_history_type_0_item_data
                    )

                    conversation_history_type_0.append(conversation_history_type_0_item)

                return conversation_history_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, list["QueryRequestConversationHistoryType0Item"]], data)

        conversation_history = _parse_conversation_history(d.pop("conversation_history", UNSET))

        def _parse_history_turns(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        history_turns = _parse_history_turns(d.pop("history_turns", UNSET))

        query_request = cls(
            query=query,
            mode=mode,
            only_need_context=only_need_context,
            only_need_prompt=only_need_prompt,
            response_type=response_type,
            top_k=top_k,
            max_token_for_text_unit=max_token_for_text_unit,
            max_token_for_global_context=max_token_for_global_context,
            max_token_for_local_context=max_token_for_local_context,
            hl_keywords=hl_keywords,
            ll_keywords=ll_keywords,
            conversation_history=conversation_history,
            history_turns=history_turns,
        )

        query_request.additional_properties = d
        return query_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class DocStatus(str, Enum):
    FAILED = "failed"
    PENDING = "pending"
    PROCESSED = "processed"
    PROCESSING = "processing"

    def __str__(self) -> str:
        return str(self.value)



"""
Configuration module for LightRAG MCP server.
"""

import argparse
from dataclasses import dataclass

DEFAULT_HOST = "localhost"
DEFAULT_PORT = 9621
DEFAULT_API_KEY = ""

DEFAULT_MCP_TRANSPORT = "stdio"
DEFAULT_MCP_HOST = "127.0.0.1"
DEFAULT_MCP_PORT = 8000
DEFAULT_MCP_STREAMABLE_HTTP_PATH = "/mcp"


@dataclass(frozen=True)
class LightRAGSettings:
    """Settings for connecting to the LightRAG API server."""

    host: str
    port: int
    api_key: str

    @property
    def base_url(self) -> str:
        return f"http://{self.host}:{self.port}"


@dataclass(frozen=True)
class MCPSettings:
    """Settings for MCP transport and HTTP server."""

    transport: str
    host: str
    port: int
    streamable_http_path: str
    stateless_http: bool
    json_response: bool

    @property
    def streamable_http_url(self) -> str:
        return f"http://{self.host}:{self.port}{self.streamable_http_path}"


def _normalize_path(path: str) -> str:
    if not path.startswith("/"):
        return f"/{path}"
    return path


def parse_args():
    """Parse command line arguments for LightRAG MCP server."""
    parser = argparse.ArgumentParser(description="LightRAG MCP Server")
    parser.add_argument(
        "--host",
        default=DEFAULT_HOST,
        help=f"LightRAG API host (default: {DEFAULT_HOST})",
    )
    parser.add_argument(
        "--port",
        type=int,
        default=DEFAULT_PORT,
        help=f"LightRAG API port (default: {DEFAULT_PORT})",
    )
    parser.add_argument("--api-key", default=DEFAULT_API_KEY, help="LightRAG API key (optional)")
    parser.add_argument(
        "--mcp-transport",
        choices=["stdio", "streamable-http"],
        default=DEFAULT_MCP_TRANSPORT,
        help=f"MCP transport (default: {DEFAULT_MCP_TRANSPORT})",
    )
    parser.add_argument(
        "--mcp-host",
        default=DEFAULT_MCP_HOST,
        help=f"MCP HTTP host (default: {DEFAULT_MCP_HOST})",
    )
    parser.add_argument(
        "--mcp-port",
        type=int,
        default=DEFAULT_MCP_PORT,
        help=f"MCP HTTP port (default: {DEFAULT_MCP_PORT})",
    )
    parser.add_argument(
        "--mcp-streamable-http-path",
        default=DEFAULT_MCP_STREAMABLE_HTTP_PATH,
        help=(
            "Streamable HTTP path for MCP endpoint "
            f"(default: {DEFAULT_MCP_STREAMABLE_HTTP_PATH})"
        ),
    )
    parser.add_argument(
        "--mcp-stateless-http",
        action="store_true",
        help="Enable stateless Streamable HTTP mode (new session per request)",
    )
    parser.add_argument(
        "--mcp-json-response",
        action="store_true",
        help="Return JSON responses instead of SSE for Streamable HTTP",
    )
    return parser.parse_args()


args = parse_args()

LIGHTRAG = LightRAGSettings(
    host=args.host,
    port=args.port,
    api_key=args.api_key,
)
MCP = MCPSettings(
    transport=args.mcp_transport,
    host=args.mcp_host,
    port=args.mcp_port,
    streamable_http_path=_normalize_path(args.mcp_streamable_http_path),
    stateless_http=args.mcp_stateless_http,
    json_response=args.mcp_json_response,
)

LIGHTRAG_API_HOST = LIGHTRAG.host
LIGHTRAG_API_PORT = LIGHTRAG.port
LIGHTRAG_API_KEY = LIGHTRAG.api_key
LIGHTRAG_API_BASE_URL = LIGHTRAG.base_url

MCP_TRANSPORT = MCP.transport
MCP_HOST = MCP.host
MCP_PORT = MCP.port
MCP_STREAMABLE_HTTP_PATH = MCP.streamable_http_path
MCP_STATELESS_HTTP = MCP.stateless_http
MCP_JSON_RESPONSE = MCP.json_response
MCP_STREAMABLE_HTTP_URL = MCP.streamable_http_url



"""
Entry point for LightRAG MCP server.
"""

import logging
import sys

from lightrag_mcp import config
from lightrag_mcp.server import mcp

logging.basicConfig(
    level=logging.INFO,
    format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
    handlers=[
        logging.StreamHandler(sys.stdout),
    ],
)
logger = logging.getLogger(__name__)


def main():
    """Main function for server startup."""
    try:
        log_level = getattr(logging, "INFO")
        logging.getLogger().setLevel(log_level)

        logger.info("Starting LightRAG MCP server")
        logger.info(
            "LightRAG API server is expected to be already running and available at: "
            f"{config.LIGHTRAG.base_url}"
        )
        logger.info(f"MCP transport: {config.MCP_TRANSPORT}")
        if config.MCP_TRANSPORT == "streamable-http":
            logger.info(f"Streamable HTTP endpoint: {config.MCP_STREAMABLE_HTTP_URL}")
            if config.MCP_STATELESS_HTTP:
                logger.info("Streamable HTTP mode: stateless")
            if config.MCP_JSON_RESPONSE:
                logger.info("Streamable HTTP mode: json_response")

        if config.LIGHTRAG.api_key:
            logger.info("API key is configured")
        else:
            logger.warning("No API key provided")

        mcp.run(transport=config.MCP_TRANSPORT)

    except KeyboardInterrupt:
        logger.info("Server stopped by user")
    except Exception as e:
        logger.exception(f"Error starting server: {str(e)}")
        sys.exit(1)


if __name__ == "__main__":
    main()



from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="EntityRequest")


@_attrs_define
class EntityRequest:
    """
    Attributes:
        entity_type (str): Type of the entity
        description (str): Description of the entity
        source_id (str): Source ID of the entity
    """

    entity_type: str
    description: str
    source_id: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        entity_type = self.entity_type

        description = self.description

        source_id = self.source_id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "entity_type": entity_type,
                "description": description,
                "source_id": source_id,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        entity_type = d.pop("entity_type")

        description = d.pop("description")

        source_id = d.pop("source_id")

        entity_request = cls(
            entity_type=entity_type,
            description=description,
            source_id=source_id,
        )

        entity_request.additional_properties = d
        return entity_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="BodyLoginLoginPost")


@_attrs_define
class BodyLoginLoginPost:
    """
    Attributes:
        username (str):
        password (str):
        grant_type (Union[None, Unset, str]):
        scope (Union[Unset, str]):  Default: ''.
        client_id (Union[None, Unset, str]):
        client_secret (Union[None, Unset, str]):
    """

    username: str
    password: str
    grant_type: Union[None, Unset, str] = UNSET
    scope: Union[Unset, str] = ""
    client_id: Union[None, Unset, str] = UNSET
    client_secret: Union[None, Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        username = self.username

        password = self.password

        grant_type: Union[None, Unset, str]
        if isinstance(self.grant_type, Unset):
            grant_type = UNSET
        else:
            grant_type = self.grant_type

        scope = self.scope

        client_id: Union[None, Unset, str]
        if isinstance(self.client_id, Unset):
            client_id = UNSET
        else:
            client_id = self.client_id

        client_secret: Union[None, Unset, str]
        if isinstance(self.client_secret, Unset):
            client_secret = UNSET
        else:
            client_secret = self.client_secret

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "username": username,
                "password": password,
            }
        )
        if grant_type is not UNSET:
            field_dict["grant_type"] = grant_type
        if scope is not UNSET:
            field_dict["scope"] = scope
        if client_id is not UNSET:
            field_dict["client_id"] = client_id
        if client_secret is not UNSET:
            field_dict["client_secret"] = client_secret

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        username = d.pop("username")

        password = d.pop("password")

        def _parse_grant_type(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        grant_type = _parse_grant_type(d.pop("grant_type", UNSET))

        scope = d.pop("scope", UNSET)

        def _parse_client_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        client_id = _parse_client_id(d.pop("client_id", UNSET))

        def _parse_client_secret(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        client_secret = _parse_client_secret(d.pop("client_secret", UNSET))

        body_login_login_post = cls(
            username=username,
            password=password,
            grant_type=grant_type,
            scope=scope,
            client_id=client_id,
            client_secret=client_secret,
        )

        body_login_login_post.additional_properties = d
        return body_login_login_post

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.doc_status import DocStatus
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.doc_status_response_metadata_type_0 import DocStatusResponseMetadataType0


T = TypeVar("T", bound="DocStatusResponse")


@_attrs_define
class DocStatusResponse:
    """
    Attributes:
        id (str):
        content_summary (str):
        content_length (int):
        status (DocStatus): Document processing status
        created_at (str):
        updated_at (str):
        file_path (str):
        chunks_count (Union[None, Unset, int]):
        error (Union[None, Unset, str]):
        metadata (Union['DocStatusResponseMetadataType0', None, Unset]):
    """

    id: str
    content_summary: str
    content_length: int
    status: DocStatus
    created_at: str
    updated_at: str
    file_path: str
    chunks_count: Union[None, Unset, int] = UNSET
    error: Union[None, Unset, str] = UNSET
    metadata: Union["DocStatusResponseMetadataType0", None, Unset] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.doc_status_response_metadata_type_0 import DocStatusResponseMetadataType0

        id = self.id

        content_summary = self.content_summary

        content_length = self.content_length

        status = self.status.value

        created_at = self.created_at

        updated_at = self.updated_at

        file_path = self.file_path

        chunks_count: Union[None, Unset, int]
        if isinstance(self.chunks_count, Unset):
            chunks_count = UNSET
        else:
            chunks_count = self.chunks_count

        error: Union[None, Unset, str]
        if isinstance(self.error, Unset):
            error = UNSET
        else:
            error = self.error

        metadata: Union[None, Unset, dict[str, Any]]
        if isinstance(self.metadata, Unset):
            metadata = UNSET
        elif isinstance(self.metadata, DocStatusResponseMetadataType0):
            metadata = self.metadata.to_dict()
        else:
            metadata = self.metadata

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "content_summary": content_summary,
                "content_length": content_length,
                "status": status,
                "created_at": created_at,
                "updated_at": updated_at,
                "file_path": file_path,
            }
        )
        if chunks_count is not UNSET:
            field_dict["chunks_count"] = chunks_count
        if error is not UNSET:
            field_dict["error"] = error
        if metadata is not UNSET:
            field_dict["metadata"] = metadata

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.doc_status_response_metadata_type_0 import DocStatusResponseMetadataType0

        d = dict(src_dict)
        id = d.pop("id")

        content_summary = d.pop("content_summary")

        content_length = d.pop("content_length")

        status = DocStatus(d.pop("status"))

        created_at = d.pop("created_at")

        updated_at = d.pop("updated_at")

        file_path = d.pop("file_path")

        def _parse_chunks_count(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        chunks_count = _parse_chunks_count(d.pop("chunks_count", UNSET))

        def _parse_error(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        error = _parse_error(d.pop("error", UNSET))

        def _parse_metadata(data: object) -> Union["DocStatusResponseMetadataType0", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                metadata_type_0 = DocStatusResponseMetadataType0.from_dict(data)

                return metadata_type_0
            except:  # noqa: E722
                pass
            return cast(Union["DocStatusResponseMetadataType0", None, Unset], data)

        metadata = _parse_metadata(d.pop("metadata", UNSET))

        doc_status_response = cls(
            id=id,
            content_summary=content_summary,
            content_length=content_length,
            status=status,
            created_at=created_at,
            updated_at=updated_at,
            file_path=file_path,
            chunks_count=chunks_count,
            error=error,
            metadata=metadata,
        )

        doc_status_response.additional_properties = d
        return doc_status_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""
Main module for LightRAG MCP server.
"""

import asyncio
import logging
from collections.abc import AsyncIterator
from contextlib import asynccontextmanager
from dataclasses import dataclass
from typing import Any, Callable, Dict, List, Union, cast

from mcp.server.fastmcp import Context, FastMCP
from pydantic import Field

from lightrag_mcp import config
from lightrag_mcp.lightrag_client import LightRAGClient

logger = logging.getLogger(__name__)


def format_response(result: Any, is_error: bool = False) -> Dict[str, Any]:
    """
    Formats response in standard format.

    Args:
        result: Operation result
        is_error: Error flag

    Returns:
        Dict[str, Any]: Standardized response
    """
    if is_error:
        if isinstance(result, str):
            return {"status": "error", "error": result}
        return {"status": "error", "error": str(result)}

    # If result is already a dictionary, return it wrapped
    if isinstance(result, dict):
        return {"status": "success", "response": result}

    # If result has dict() method or __dict__, use it
    if hasattr(result, "dict") and callable(getattr(result, "dict")):
        return {"status": "success", "response": result.dict()}
    if hasattr(result, "__dict__"):
        return {"status": "success", "response": result.__dict__}
    if hasattr(result, "to_dict") and callable(getattr(result, "to_dict")):
        return {"status": "success", "response": result.to_dict()}

    # In other cases, convert to string
    return {"status": "success", "response": str(result)}


@dataclass
class AppContext:
    """Application context with typed resources."""

    lightrag_client: LightRAGClient


@asynccontextmanager
async def app_lifespan(server: FastMCP) -> AsyncIterator[AppContext]:
    """
    Manages application lifecycle with typed context.
    Initializes LightRAG API client at startup and closes it at shutdown.
    """
    lightrag_client = LightRAGClient(
        base_url=config.LIGHTRAG.base_url,
        api_key=config.LIGHTRAG.api_key,
    )

    try:
        yield AppContext(lightrag_client=lightrag_client)
    finally:
        await lightrag_client.close()
        logger.info("LightRAG MCP Server stopped")


mcp = FastMCP(
    "LightRAG MCP Server",
    lifespan=app_lifespan,
    host=config.MCP_HOST,
    port=config.MCP_PORT,
    streamable_http_path=config.MCP_STREAMABLE_HTTP_PATH,
    stateless_http=config.MCP_STATELESS_HTTP,
    json_response=config.MCP_JSON_RESPONSE,
)


async def execute_lightrag_operation(
    operation_name: str, operation_func: Callable, ctx: Context
) -> Dict[str, Any]:
    """
    Universal wrapper function for executing operations with LightRAG API.

    Automatically handles:
    - Getting client from context
    - Type casting
    - Exception handling
    - Response formatting

    Args:
        operation_name: Operation name for logging
        operation_func: Function to execute that takes client as first argument

    Returns:
        Dict[str, Any]: Formatted response
    """
    try:
        if not ctx or not ctx.request_context or not ctx.request_context.lifespan_context:
            return format_response(
                f"Error: Request context is not available for {operation_name}", is_error=True
            )

        app_ctx = cast(AppContext, ctx.request_context.lifespan_context)
        client = app_ctx.lightrag_client

        logger.info(f"Executing operation: {operation_name}")
        result = await operation_func(client)

        return format_response(result)
    except Exception as e:
        logger.exception(f"Error during {operation_name}: {str(e)}")
        return format_response(str(e), is_error=True)


# === MCP Tools ===


@mcp.tool(name="query_document", description="Execute a query to documents through LightRAG API")
async def query_document(
    ctx: Context,
    query: str = Field(description="Query text"),
    mode: str = Field(
        description="Search mode (mix, semantic, keyword, global, hybrid, local, naive)",
        default="mix",
    ),
    top_k: int = Field(description="Number of results", default=60),
    only_need_context: bool = Field(
        description="Return only context without LLM response", default=False
    ),
    only_need_prompt: bool = Field(
        description="If True, returns only generated prompt without creating a response",
        default=False,
    ),
    response_type: str = Field(
        description="Определяет формат ответа. Примеры: 'Multiple Paragraphs', 'Single Paragraph', 'Bullet Points'",
        default="Multiple Paragraphs",
    ),
    max_token_for_text_unit: int = Field(
        description="Maximum tokens for each text fragment", default=4096
    ),
    max_token_for_global_context: int = Field(
        description="Maximum tokens for global context", default=4096
    ),
    max_token_for_local_context: int = Field(
        description="Maximum tokens for local context", default=4096
    ),
    hl_keywords: list[str] = Field(
        description="List of high-level keywords for prioritization", default=[]
    ),
    ll_keywords: list[str] = Field(
        description="List of low-level keywords for search refinement", default=[]
    ),
    history_turns: int = Field(
        description="Number of conversation turns in response context", default=10
    ),
) -> Dict[str, Any]:
    async def _operation(client: LightRAGClient) -> Any:
        return await client.query(
            query_text=query,
            mode=mode,
            top_k=top_k,
            only_need_context=only_need_context,
            only_need_prompt=only_need_prompt,
            response_type=response_type,
            max_token_for_text_unit=max_token_for_text_unit,
            max_token_for_global_context=max_token_for_global_context,
            max_token_for_local_context=max_token_for_local_context,
            hl_keywords=hl_keywords,
            ll_keywords=ll_keywords,
            history_turns=history_turns,
        )

    return await execute_lightrag_operation(
        operation_name=f"query execution: {query[:50]}...",
        operation_func=_operation,
        ctx=ctx,
    )


@mcp.tool(name="insert_document", description="Add text directly to LightRAG storage")
async def insert_document(
    ctx: Context,
    text: Union[str, List[str]] = Field(description="Text or list of texts to add"),
) -> Dict[str, Any]:
    async def _operation(client: LightRAGClient) -> Any:
        return await client.insert_text(text=text)

    return await execute_lightrag_operation(
        operation_name="text insertion",
        operation_func=_operation,
        ctx=ctx,
    )


@mcp.tool(
    name="upload_document",
    description="Upload document from file to LightRAG's /inputs directory and start indexing",
)
async def upload_document(
    ctx: Context,
    file_path: str = Field(description="Path to file for upload"),
) -> Dict[str, Any]:
    async def _operation(client: LightRAGClient) -> Any:
        return await client.upload_document(file_path=file_path)

    return await execute_lightrag_operation(
        operation_name=f"file upload: {file_path}",
        operation_func=_operation,
        ctx=ctx,
    )


@mcp.tool(name="insert_file", description="Add document from file to LightRAG")
async def insert_file(
    ctx: Context,
    file_path: str = Field(description="Path to file for upload"),
) -> Dict[str, Any]:
    async def _operation(client: LightRAGClient) -> Any:
        return await client.insert_file(file_path=file_path)

    return await execute_lightrag_operation(
        operation_name=f"file insertion: {file_path}",
        operation_func=_operation,
        ctx=ctx,
    )


@mcp.tool(name="insert_batch", description="Add batch of documents from directory to LightRAG")
async def insert_batch(
    ctx: Context,
    directory_path: str = Field(description="Path to directory with files to add"),
    recursive: bool = Field(description="Recursive addition. Defaults to False", default=False),
    depth: int = Field(description="Recursion depth. Defaults to 1", default=1),
    include_only: list[str] = Field(
        description="""
        List of regexp to specify files to include. 
        Files not matching any regexp will be skipped. 
        Defaults to [] (all files). 
        Either include_only or ignore_files must be specified, not both.
        """,
        default=[],
    ),
    ignore_files: list[str] = Field(
        description="""
        List of regexp to exclude files from batch insertion.
        Defaults to [] (no files are excluded).
        Either ignore_files or include_only must be specified, not both.
        """,
        default=[],
    ),
    ignore_directories: list[str] = Field(
        description="""
        List of regexp to exclude directories from batch insertion.
        Defaults to [] (no directories are excluded).
        """,
        default=[],
    ),
) -> Dict[str, Any]:
    async def _operation(client: LightRAGClient) -> Any:
        return await client.insert_batch(
            directory_path=directory_path,
            recursive=recursive,
            depth=depth,
            include_only=include_only,
            ignore_directories=ignore_directories,
            ignore_files=ignore_files,
        )

    return await execute_lightrag_operation(
        operation_name=f"batch insertion from directory: {directory_path}",
        operation_func=_operation,
        ctx=ctx,
    )


@mcp.tool(
    name="scan_for_new_documents",
    description="Start scanning LightRAG /inputs directory for new documents",
)
async def scan_for_new_documents(ctx: Context) -> Dict[str, Any]:
    async def _operation(client: LightRAGClient) -> Any:
        return await client.scan_for_new_documents()

    return await execute_lightrag_operation(
        operation_name="scanning for new documents",
        operation_func=_operation,
        ctx=ctx,
    )


@mcp.tool(name="get_documents", description="Get list of all uploaded documents")
async def get_documents(ctx: Context) -> Dict[str, Any]:
    async def _operation(client: LightRAGClient) -> Any:
        return await client.get_documents()

    return await execute_lightrag_operation(
        operation_name="getting documents list",
        operation_func=_operation,
        ctx=ctx,
    )


@mcp.tool(name="get_pipeline_status", description="Get status of document processing in pipeline")
async def get_pipeline_status(ctx: Context) -> Dict[str, Any]:
    async def _operation(client: LightRAGClient) -> Any:
        return await client.get_pipeline_status()

    return await execute_lightrag_operation(
        operation_name="getting pipeline status",
        operation_func=_operation,
        ctx=ctx,
    )


@mcp.tool(
    name="get_graph_labels",
    description="Get labels (node and relationship types) from knowledge graph",
)
async def get_graph_labels(ctx: Context) -> Dict[str, Any]:
    async def _operation(client: LightRAGClient) -> Any:
        return await client.get_graph_labels()

    return await execute_lightrag_operation(
        operation_name="getting graph labels",
        operation_func=_operation,
        ctx=ctx,
    )


@mcp.tool(name="check_lightrag_health", description="Check LightRAG API status")
async def check_lightrag_health(ctx: Context) -> Dict[str, Any]:
    async def _operation(client: LightRAGClient) -> Any:
        result = await client.get_health()
        if isinstance(result, dict) and "status" in result:
            logger.info(f"LightRAG API returned status: {result['status']}")
        return result

    return await execute_lightrag_operation(
        operation_name="health check",
        operation_func=_operation,
        ctx=ctx,
    )


@mcp.tool(
    name="merge_entities",
    description="Merge multiple entities into one with relationship migration",
)
async def merge_entities(
    ctx: Context,
    source_entities: List[str] = Field(description="List of entity names to merge"),
    target_entity: str = Field(description="Target entity name"),
    merge_strategy: Dict[str, str] = Field(
        description="Property merge strategy. Supported strategies: 'concatenate' (concatenate text values), 'keep_first' (keep first non-empty value), 'keep_last' (keep last non-empty value), 'join_unique' (join unique values for delimited fields). Example: {'description': 'concatenate', 'entity_type': 'keep_first'}",
        default_factory=dict,
    ),
) -> Dict[str, Any]:
    async def _operation(client: LightRAGClient) -> Any:
        return await client.merge_entities(
            source_entities=source_entities,
            target_entity=target_entity,
            merge_strategy=merge_strategy,
        )

    return await execute_lightrag_operation(
        operation_name=f"entity merging: {', '.join(source_entities)} -> {target_entity}",
        operation_func=_operation,
        ctx=ctx,
    )


@mcp.tool(name="create_entities", description="Create multiple entities in knowledge graph")
async def create_entities(
    ctx: Context,
    entities: List[Dict[str, Any]] = Field(
        description="""
        List of entity dictionaries, each containing:
        - entity_name (str): Name of the entity
        - entity_type (str): Type of the entity
        - description (str): Description of the entity
        - source_id (str): Source ID (document)
        
        Example:
        [
            {
                "entity_name": "Python",
                "entity_type": "PROGRAMMING_LANGUAGE",
                "description": "Python is a high-level programming language",
                "source_id": "doc123"
            },
            {
                "entity_name": "JavaScript",
                "entity_type": "PROGRAMMING_LANGUAGE",
                "description": "JavaScript is a scripting language",
                "source_id": "doc456"
            }
        ]
        """
    ),
) -> Dict[str, Any]:
    """
    Create multiple entities in the knowledge graph in parallel.

    Args:
        entities: List of entity dictionaries with required fields:
                entity_name,
                entity_type,
                description,
                source_id

    Returns:
        Dictionary with creation results for each entity
    """

    async def _create_entity(client: LightRAGClient, entity_data: Dict[str, Any]) -> Dict[str, Any]:
        entity_name = entity_data.get("entity_name")
        entity_type = entity_data.get("entity_type")
        description = entity_data.get("description")
        source_id = entity_data.get("source_id")

        if not all([entity_name, entity_type, description, source_id]):
            return {
                "entity_name": str(entity_name or "unknown"),
                "status": "error",
                "error": "Missing required fields",
            }

        try:
            result = await client.create_entity(
                entity_name=str(entity_name),
                entity_type=str(entity_type),
                description=str(description),
                source_id=str(source_id),
            )
            return {"entity_name": str(entity_name), "status": "success", "result": result}
        except Exception as e:
            return {"entity_name": str(entity_name), "status": "error", "error": str(e)}

    async def _operation(client: LightRAGClient) -> Any:
        entity_names = [entity.get("entity_name", "unknown") for entity in entities]
        entity_names_str = ", ".join(entity_names[:5])
        if len(entity_names) > 5:
            entity_names_str += f" and {len(entity_names) - 5} more"

        # Create tasks for parallel execution
        tasks = [_create_entity(client, entity_data) for entity_data in entities]
        results = await asyncio.gather(*tasks)

        return {
            "total": len(entities),
            "successful": sum(1 for r in results if r.get("status") == "success"),
            "failed": sum(1 for r in results if r.get("status") == "error"),
            "results": results,
        }

    return await execute_lightrag_operation(
        operation_name=f"bulk entity creation: {len(entities)} entities",
        operation_func=_operation,
        ctx=ctx,
    )


@mcp.tool(
    name="delete_by_entities", description="Delete multiple entities from knowledge graph by name"
)
async def delete_by_entities(
    ctx: Context,
    entity_names: List[str] = Field(
        description="""
        List of entity names to delete.
        
        Example:
        ["Python", "JavaScript", "TypeScript"]
        """
    ),
) -> Dict[str, Any]:
    """
    Delete multiple entities from the knowledge graph in parallel.

    Args:
        entity_names: List of entity names to delete

    Returns:
        Dictionary with deletion results for each entity
    """

    async def _delete_entity(client: LightRAGClient, entity_name: str) -> Dict[str, Any]:
        try:
            result = await client.delete_by_entity(entity_name=entity_name)
            return {"entity_name": entity_name, "status": "success", "result": result}
        except Exception as e:
            return {"entity_name": entity_name, "status": "error", "error": str(e)}

    async def _operation(client: LightRAGClient) -> Any:
        entity_names_str = ", ".join(entity_names[:5])
        if len(entity_names) > 5:
            entity_names_str += f" and {len(entity_names) - 5} more"

        # Create tasks for parallel execution
        tasks = [_delete_entity(client, entity_name) for entity_name in entity_names]
        results = await asyncio.gather(*tasks)

        return {
            "total": len(entity_names),
            "successful": sum(1 for r in results if r.get("status") == "success"),
            "failed": sum(1 for r in results if r.get("status") == "error"),
            "results": results,
        }

    return await execute_lightrag_operation(
        operation_name=f"bulk entity deletion: {len(entity_names)} entities",
        operation_func=_operation,
        ctx=ctx,
    )


@mcp.tool(
    name="delete_by_doc_ids",
    description="Delete all entities and relationships associated with multiple documents",
)
async def delete_by_doc_ids(
    ctx: Context,
    doc_ids: List[str] = Field(
        description="""
        List of document IDs to delete entities from.
        
        Example:
        ["doc123", "doc456", "doc789"]
        """
    ),
) -> Dict[str, Any]:
    """
    Delete entities and relationships associated with multiple documents in parallel.

    Args:
        doc_ids: List of document IDs to delete entities from

    Returns:
        Dictionary with deletion results for each document ID
    """

    async def _delete_by_doc_id(client: LightRAGClient, doc_id: str) -> Dict[str, Any]:
        try:
            result = await client.delete_by_doc_id(doc_id=doc_id)
            return {"doc_id": doc_id, "status": "success", "result": result}
        except Exception as e:
            return {"doc_id": doc_id, "status": "error", "error": str(e)}

    async def _operation(client: LightRAGClient) -> Any:
        doc_ids_str = ", ".join(doc_ids[:5])
        if len(doc_ids) > 5:
            doc_ids_str += f" and {len(doc_ids) - 5} more"

        # Create tasks for parallel execution
        tasks = [_delete_by_doc_id(client, doc_id) for doc_id in doc_ids]
        results = await asyncio.gather(*tasks)

        return {
            "total": len(doc_ids),
            "successful": sum(1 for r in results if r.get("status") == "success"),
            "failed": sum(1 for r in results if r.get("status") == "error"),
            "results": results,
        }

    return await execute_lightrag_operation(
        operation_name=f"bulk deletion by document IDs: {len(doc_ids)} documents",
        operation_func=_operation,
        ctx=ctx,
    )


@mcp.tool(name="edit_entities", description="Edit multiple existing entities in knowledge graph")
async def edit_entities(
    ctx: Context,
    entities: List[Dict[str, Any]] = Field(
        description="""
        List of entity dictionaries to edit, each containing:
        - entity_name (str): Name of the entity to edit
        - entity_type (str): New entity type
        - description (str): New entity description
        - source_id (str): Source ID (document)
        
        Example:
        [
            {
                "entity_name": "Python",
                "entity_type": "PROGRAMMING_LANGUAGE",
                "description": "Updated description for Python",
                "source_id": "doc123"
            },
            {
                "entity_name": "JavaScript",
                "entity_type": "PROGRAMMING_LANGUAGE",
                "description": "Updated description for JavaScript",
                "source_id": "doc456"
            }
        ]
        """
    ),
) -> Dict[str, Any]:
    """
    Edit multiple entities in the knowledge graph in parallel.

    Args:
        entities: List of entity dictionaries with required fields:
                entity_name,
                entity_type,
                description,
                source_id

    Returns:
        Dictionary with edit results for each entity
    """

    async def _edit_entity(client: LightRAGClient, entity_data: Dict[str, Any]) -> Dict[str, Any]:
        entity_name = entity_data.get("entity_name")
        entity_type = entity_data.get("entity_type")
        description = entity_data.get("description")
        source_id = entity_data.get("source_id")

        if not all([entity_name, entity_type, description, source_id]):
            return {
                "entity_name": str(entity_name or "unknown"),
                "status": "error",
                "error": "Missing required fields",
            }

        try:
            result = await client.edit_entity(
                entity_name=str(entity_name),
                entity_type=str(entity_type),
                description=str(description),
                source_id=str(source_id),
            )
            return {"entity_name": str(entity_name), "status": "success", "result": result}
        except Exception as e:
            return {"entity_name": str(entity_name), "status": "error", "error": str(e)}

    async def _operation(client: LightRAGClient) -> Any:
        entity_names = [entity.get("entity_name", "unknown") for entity in entities]
        entity_names_str = ", ".join(entity_names[:5])
        if len(entity_names) > 5:
            entity_names_str += f" and {len(entity_names) - 5} more"

        # Create tasks for parallel execution
        tasks = [_edit_entity(client, entity_data) for entity_data in entities]
        results = await asyncio.gather(*tasks)

        return {
            "total": len(entities),
            "successful": sum(1 for r in results if r.get("status") == "success"),
            "failed": sum(1 for r in results if r.get("status") == "error"),
            "results": results,
        }

    return await execute_lightrag_operation(
        operation_name=f"bulk entity editing: {len(entities)} entities",
        operation_func=_operation,
        ctx=ctx,
    )


@mcp.tool(
    name="create_relations",
    description="Create multiple relationships between entities in knowledge graph",
)
async def create_relations(
    ctx: Context,
    relations: List[Dict[str, Any]] = Field(
        description="""
        List of relation dictionaries to create, each containing:
        - source (str): Source entity name
        - target (str): Target entity name
        - description (str): Relationship description
        - keywords (str): Keywords for relationship
        - source_id (str, optional): Source ID (document)
        - weight (float, optional): Relationship weight
        
        Example:
        [
            {
                "source": "Python",
                "target": "Django",
                "description": "Python is used to build Django",
                "keywords": "framework, web development",
                "source_id": "doc123",
                "weight": 0.8
            },
            {
                "source": "JavaScript",
                "target": "React",
                "description": "JavaScript is used to build React",
                "keywords": "framework, frontend",
                "source_id": "doc456",
                "weight": 0.9
            }
        ]
        """
    ),
) -> Dict[str, Any]:
    """
    Create multiple relationships between entities in the knowledge graph in parallel.

    Args:
        relations: List of relation dictionaries with required fields:
                source, target, description, keywords
                and optional fields: source_id, weight

    Returns:
        Dictionary with creation results for each relationship
    """

    async def _create_relation(
        client: LightRAGClient, relation_data: Dict[str, Any]
    ) -> Dict[str, Any]:
        source = relation_data.get("source")
        target = relation_data.get("target")
        description = relation_data.get("description")
        keywords = relation_data.get("keywords")
        source_id = relation_data.get("source_id")
        weight = relation_data.get("weight")

        if not all([source, target, description, keywords]):
            return {
                "relation": f"{str(source or 'unknown')} -> {str(target or 'unknown')}",
                "status": "error",
                "error": "Missing required fields",
            }

        try:
            result = await client.create_relation(
                source=str(source),
                target=str(target),
                description=str(description),
                keywords=str(keywords),
                source_id=str(source_id) if source_id else None,
                weight=float(weight) if weight is not None else None,
            )
            return {"relation": f"{source} -> {target}", "status": "success", "result": result}
        except Exception as e:
            return {"relation": f"{source} -> {target}", "status": "error", "error": str(e)}

    async def _operation(client: LightRAGClient) -> Any:
        relation_strs = [
            f"{r.get('source', 'unknown')} -> {r.get('target', 'unknown')}" for r in relations
        ]
        relation_strs_sample = ", ".join(relation_strs[:5])
        if len(relation_strs) > 5:
            relation_strs_sample += f" and {len(relation_strs) - 5} more"

        # Create tasks for parallel execution
        tasks = [_create_relation(client, relation_data) for relation_data in relations]
        results = await asyncio.gather(*tasks)

        return {
            "total": len(relations),
            "successful": sum(1 for r in results if r.get("status") == "success"),
            "failed": sum(1 for r in results if r.get("status") == "error"),
            "results": results,
        }

    return await execute_lightrag_operation(
        operation_name=f"bulk relationship creation: {len(relations)} relationships",
        operation_func=_operation,
        ctx=ctx,
    )


@mcp.tool(
    name="edit_relations",
    description="Edit multiple relationships between entities in knowledge graph",
)
async def edit_relations(
    ctx: Context,
    relations: List[Dict[str, Any]] = Field(
        description="""
        List of relation dictionaries to edit, each containing:
        - source (str): Source entity name
        - target (str): Target entity name
        - description (str): New relationship description
        - keywords (str): New keywords for relationship
        - relation_type (str): Relationship type
        - source_id (str, optional): Source ID (document)
        - weight (float, optional): New relationship weight
        
        Example:
        [
            {
                "source": "Python",
                "target": "Django",
                "description": "Updated description for Python-Django relation",
                "keywords": "framework, web development, updated",
                "relation_type": "USES",
                "source_id": "doc123",
                "weight": 0.85
            },
            {
                "source": "JavaScript",
                "target": "React",
                "description": "Updated description for JavaScript-React relation",
                "keywords": "framework, frontend, updated",
                "relation_type": "USES",
                "source_id": "doc456",
                "weight": 0.95
            }
        ]
        """
    ),
) -> Dict[str, Any]:
    """
    Edit multiple relationships between entities in the knowledge graph in parallel.

    Args:
        relations: List of relation dictionaries with required fields:
                source, target, description, keywords, relation_type
                and optional fields: source_id, weight

    Returns:
        Dictionary with edit results for each relationship
    """

    async def _edit_relation(
        client: LightRAGClient, relation_data: Dict[str, Any]
    ) -> Dict[str, Any]:
        source = relation_data.get("source")
        target = relation_data.get("target")
        description = relation_data.get("description")
        keywords = relation_data.get("keywords")
        relation_type = relation_data.get("relation_type")
        source_id = relation_data.get("source_id")
        weight = relation_data.get("weight")

        if not all([source, target, description, keywords, relation_type]):
            return {
                "relation": f"{str(source or 'unknown')} -> {str(target or 'unknown')}",
                "status": "error",
                "error": "Missing required fields",
            }

        try:
            result = await client.edit_relation(
                source=str(source),
                target=str(target),
                description=str(description),
                keywords=str(keywords),
                relation_type=str(relation_type),
                source_id=str(source_id) if source_id else None,
                weight=float(weight) if weight is not None else None,
            )
            return {"relation": f"{source} -> {target}", "status": "success", "result": result}
        except Exception as e:
            return {"relation": f"{source} -> {target}", "status": "error", "error": str(e)}

    async def _operation(client: LightRAGClient) -> Any:
        relation_strs = [
            f"{r.get('source', 'unknown')} -> {r.get('target', 'unknown')}" for r in relations
        ]
        relation_strs_sample = ", ".join(relation_strs[:5])
        if len(relation_strs) > 5:
            relation_strs_sample += f" and {len(relation_strs) - 5} more"

        # Create tasks for parallel execution
        tasks = [_edit_relation(client, relation_data) for relation_data in relations]
        results = await asyncio.gather(*tasks)

        return {
            "total": len(relations),
            "successful": sum(1 for r in results if r.get("status") == "success"),
            "failed": sum(1 for r in results if r.get("status") == "error"),
            "results": results,
        }

    return await execute_lightrag_operation(
        operation_name=f"bulk relationship editing: {len(relations)} relationships",
        operation_func=_operation,
        ctx=ctx,
    )



"""
Client for interacting with LightRAG API.
"""

import logging
import re
from pathlib import Path
from typing import Any, Awaitable, Callable, Dict, List, TypeVar, Union

from lightrag_mcp.client.light_rag_server_api_client.api.default import async_get_health
from lightrag_mcp.client.light_rag_server_api_client.api.documents import (
    async_get_documents,
    async_get_pipeline_status,
    async_insert_document,
    async_insert_file,
    async_insert_texts,
    async_scan_for_new_documents,
    async_upload_document,
)
from lightrag_mcp.client.light_rag_server_api_client.api.graph import (
    async_create_entity,
    async_create_relation,
    async_delete_by_doc_id,
    async_delete_entity,
    async_edit_entity,
    async_edit_relation,
    async_get_graph_labels,
    async_merge_entities,
)
from lightrag_mcp.client.light_rag_server_api_client.api.query import (
    async_query_document,
)
from lightrag_mcp.client.light_rag_server_api_client.client import AuthenticatedClient
from lightrag_mcp.client.light_rag_server_api_client.models import (
    BodyInsertFileDocumentsFilePost,
    BodyUploadToInputDirDocumentsUploadPost,
    DocsStatusesResponse,
    HTTPValidationError,
    InsertResponse,
    InsertTextRequest,
    InsertTextsRequest,
    PipelineStatusResponse,
    QueryRequest,
    QueryRequestMode,
    QueryResponse,
    relation_request,
    relation_response,
)
from lightrag_mcp.client.light_rag_server_api_client.models.entity_request import EntityRequest
from lightrag_mcp.client.light_rag_server_api_client.models.entity_response import EntityResponse
from lightrag_mcp.client.light_rag_server_api_client.models.merge_entities_request import (
    MergeEntitiesRequest,
)
from lightrag_mcp.client.light_rag_server_api_client.models.merge_entities_request_merge_strategy_type_0 import (
    MergeEntitiesRequestMergeStrategyType0,
)
from lightrag_mcp.client.light_rag_server_api_client.models.status_message_response import (
    StatusMessageResponse,
)
from lightrag_mcp.client.light_rag_server_api_client.types import File

from .client.light_rag_server_api_client.errors import UnexpectedStatus

logger = logging.getLogger(__name__)

T = TypeVar("T", covariant=True)
ApiFunc = Callable[..., Awaitable[Union[T, HTTPValidationError, None]]]


class LightRAGClient:
    """
    Client for interacting with LightRAG API.
    """

    def __init__(
        self,
        base_url: str,
        api_key: str,
    ):
        """
        Initialize LightRAG API client.

        Args:
            base_url (str): Base API URL.
            api_key (str): API key (token).
        """
        self.base_url = base_url
        self.api_key = api_key
        self.client = AuthenticatedClient(base_url=base_url, token=api_key, verify_ssl=False)
        logger.info(f"Initialized LightRAG API client: {base_url}")

    async def _handle_exception(self, e: Exception, operation_name: str) -> None:
        """
        Handle exceptions when calling API.

        Args:
            e: Exception
            operation_name: Operation name for logging

        Raises:
            Exception: Re-raises the exception
        """
        if isinstance(e, UnexpectedStatus):
            logger.error(f"HTTP error during {operation_name}: {e.status_code} - {e.content!r}")
        else:
            logger.error(f"Error during {operation_name}: {str(e)}")

    async def _call_api(
        self,
        api_func: Callable[..., Awaitable[Union[T, HTTPValidationError, None]]],
        operation_name: str,
        **kwargs,
    ) -> Union[T, HTTPValidationError, None]:
        """
        Universal method for calling API functions.

        Args:
            api_func: API function to call
            operation_name: Operation name for logging
            **kwargs: Additional arguments for the API function

        Returns:
            Union[T, HTTPValidationError, None]: API call result
        """
        try:
            logger.debug(f"Calling API: {operation_name}")
            result = await api_func(**kwargs)
            logger.debug(f"API call successful: {operation_name}")
            return result
        except Exception as e:
            await self._handle_exception(e, operation_name)
            raise

    async def query(
        self,
        query_text: str,
        mode: str = "mix",
        top_k: int = 10,
        only_need_context: bool = False,
        only_need_prompt: bool = False,
        response_type: str = "Multiple Paragraphs",
        max_token_for_text_unit: int = 1000,
        max_token_for_global_context: int = 1000,
        max_token_for_local_context: int = 1000,
        hl_keywords: list[str] = [],
        ll_keywords: list[str] = [],
        history_turns: int = 10,
    ) -> Union[QueryResponse, HTTPValidationError, None]:
        """
        Execute a query to LightRAG API.

        Args:
            query_text (str): Query text
            mode (str, optional): Search mode (global, hybrid, local, mix, naive). Default is "mix".
            response_type (str, optional): Response format. Default is "Multiple Paragraphs".
            top_k (int, optional): Number of results. Default is 10.
            only_need_context (bool, optional): Return only context without LLM response. Default is False.
            only_need_prompt (bool, optional): Return only generated prompt without creating a response. Default is False.
            max_token_for_text_unit (int, optional): Maximum tokens for each text fragment. Default is 1000.
            max_token_for_global_context (int, optional): Maximum tokens for global context. Default is 1000.
            max_token_for_local_context (int, optional): Maximum tokens for local context. Default is 1000.
            hl_keywords (list[str], optional): List of high-level keywords for prioritization. Default is [].
            ll_keywords (list[str], optional): List of low-level keywords for search refinement. Default is [].
            history_turns (int, optional): Number of conversation turns in response context. Default is 10.

        Returns:
            Union[QueryResponse, HTTPValidationError, None]: Query result
        """
        logger.debug(f"Executing query: {query_text[:100]}...")

        request = QueryRequest(
            query=query_text,
            mode=QueryRequestMode(mode),
            response_type=response_type,
            top_k=top_k,
            only_need_context=only_need_context,
            only_need_prompt=only_need_prompt,
            max_token_for_text_unit=max_token_for_text_unit,
            max_token_for_global_context=max_token_for_global_context,
            max_token_for_local_context=max_token_for_local_context,
            hl_keywords=hl_keywords,
            ll_keywords=ll_keywords,
            history_turns=history_turns,
        )

        return await self._call_api(
            api_func=async_query_document,
            operation_name="query execution",
            client=self.client,
            body=request,
        )

    async def insert_text(
        self,
        text: Union[str, List[str]],
    ) -> Union[InsertResponse, HTTPValidationError, None]:
        """
        Add text to LightRAG.

        Args:
            text (Union[str, List[str]]): Text or list of texts to add

        Returns:
            Union[InsertResponse, HTTPValidationError]: Operation result
        """
        logger.debug(f"Adding text: {str(text)[:100]}...")

        request: InsertTextRequest | InsertTextsRequest
        if isinstance(text, str):
            request = InsertTextRequest(text=text)
            return await self._call_api(
                api_func=async_insert_document,
                operation_name="text insertion",
                client=self.client,
                body=request,
            )
        else:
            request = InsertTextsRequest(texts=text)
            return await self._call_api(
                api_func=async_insert_texts,
                operation_name="multiple texts insertion",
                client=self.client,
                body=request,
            )

    async def upload_document(self, file_path: str) -> Union[Any, HTTPValidationError, None]:
        """
        Upload document from file to LightRAG's /input directory and start indexing.

        Args:
            file_path (str): Path to file.

        Returns:
            Union[Any, HTTPValidationError]: Operation result.
        """
        logger.debug(f"Uploading document: {file_path}")

        path = Path(file_path)
        if not path.exists():
            logger.error(f"File not found: {file_path}")
            raise FileNotFoundError(f"File not found: {file_path}")

        try:
            with open(file_path, "rb") as f:
                file_name = path.name
                upload_request = BodyUploadToInputDirDocumentsUploadPost(
                    file=File(payload=f, file_name=file_name)
                )

                return await self._call_api(
                    api_func=async_upload_document,
                    operation_name=f"file upload {file_path}",
                    client=self.client,
                    body=upload_request,
                )
        except FileNotFoundError:
            logger.error(f"Файл не найден: {file_path}")
            raise
        except Exception as e:
            await self._handle_exception(e, f"загрузке файла {file_path}")
            raise

    async def insert_file(self, file_path: str) -> Union[InsertResponse, HTTPValidationError, None]:
        """
        Add document from a file_path directly to LightRAG storage, without uploading to /input directory.

        Args:
            file_path (str): Path to file.

        Returns:
            Union[InsertResponse, HTTPValidationError]: Operation result.
        """
        logger.debug(f"Adding file: {file_path}")

        path = Path(file_path)
        if not path.exists():
            logger.error(f"File not found: {file_path}")
            raise FileNotFoundError(f"File not found: {file_path}")

        try:
            with open(file_path, "rb") as f:
                file_name = path.name
                insert_file_request = BodyInsertFileDocumentsFilePost(
                    file=File(payload=f, file_name=file_name)
                )

                return await self._call_api(
                    api_func=async_insert_file,
                    operation_name=f"file insertion {file_path}",
                    client=self.client,
                    body=insert_file_request,
                )
        except FileNotFoundError:
            logger.error(f"Файл не найден: {file_path}")
            raise
        except Exception as e:
            await self._handle_exception(e, f"добавлении файла {file_path}")
            raise

    async def insert_batch(
        self,
        directory_path: str,
        recursive: bool = False,
        depth: int = 1,
        include_only: list[str] = [],
        ignore_directories: list[str] = [],
        ignore_files: list[str] = [],
    ) -> Union[InsertResponse, HTTPValidationError, None]:
        """
        Add batch of documents from directory.

        Args:
            directory_path (str): Path to directory.
            recursive (bool, optional): Recursive addition. Defaults to False.
            depth (int, optional): Recursion depth. Defaults to 1.
            ignore_directories (list[str], optional): List of regexp to exclude directories from batch insertion. Defaults to [].
            ignore_files (list[str], optional): List of regexp to exclude files from batch insertion. Defaults to []. Either ignore_files or include_only must be specified, not both.
            include_only (list[str], optional): List of regexp to specify files to include. Defaults to []. Either include_only or ignore_files must be specified, not both.

        Returns:
            Union[InsertResponse, HTTPValidationError]: Operation result.
        """
        logger.debug(
            f"Adding batch of documents from directory: {directory_path} (recursive={recursive}, depth={depth})"
        )

        if include_only and ignore_files:
            error_message = "Cannot specify both include_only and ignore_files parameters"
            logger.error(error_message)
            raise ValueError(error_message)

        dir_path = Path(directory_path)
        if not dir_path.exists() or not dir_path.is_dir():
            logger.error(f"Directory not found: {directory_path}")
            raise FileNotFoundError(f"Directory not found: {directory_path}")

        include_patterns = [re.compile(pattern) for pattern in include_only] if include_only else []
        ignore_dir_patterns = (
            [re.compile(pattern) for pattern in ignore_directories] if ignore_directories else []
        )
        ignore_file_patterns = (
            [re.compile(pattern) for pattern in ignore_files] if ignore_files else []
        )

        def collect_file_paths(dir_path: Path, current_depth: int = 0) -> List[Path]:
            """Recursively collect file paths from directory"""
            file_paths = []
            try:
                for item in dir_path.iterdir():
                    if item.is_dir() and recursive and current_depth < depth:
                        # Check if directory should be ignored
                        dir_name = item.name
                        if any(pattern.search(dir_name) for pattern in ignore_dir_patterns):
                            logger.debug(f"Ignoring directory: {item} (matched ignore pattern)")
                            continue

                        # Process subdirectory
                        file_paths.extend(collect_file_paths(item, current_depth + 1))
                    elif item.is_file():
                        file_name = item.name

                        # Apply include_only filter if specified
                        if include_patterns:
                            if any(pattern.search(file_name) for pattern in include_patterns):
                                file_paths.append(item)
                                logger.debug(f"Including file: {item} (matched include pattern)")
                            else:
                                logger.debug(
                                    f"Skipping file: {item} (did not match any include pattern)"
                                )
                            continue

                        # Apply ignore_files filter if specified
                        if ignore_file_patterns:
                            if any(pattern.search(file_name) for pattern in ignore_file_patterns):
                                logger.debug(f"Ignoring file: {item} (matched ignore pattern)")
                                continue

                        # If we got here, the file is not filtered out
                        file_paths.append(item)
            except Exception as e:
                logger.error(f"Error collecting files from {dir_path}: {str(e)}")
            return file_paths

        try:
            file_paths = collect_file_paths(dir_path)
            logger.info(f"Found {len(file_paths)} files for processing after applying filters")

            success_count = 0
            failed_files = []

            for file_path in file_paths:
                try:
                    await self.insert_file(str(file_path))
                    success_count += 1
                    logger.debug(f"Successfully inserted file: {file_path}")
                except Exception as e:
                    logger.error(f"Error inserting file {file_path}: {str(e)}")
                    failed_files.append(str(file_path))

            if success_count == len(file_paths):
                status = "success"
                message = f"All {success_count} documents inserted successfully"
            elif success_count > 0:
                status = "partial_success"
                message = (
                    f"Successfully inserted {success_count} out of {len(file_paths)} documents"
                )
                if failed_files:
                    message += f". Failed files: {', '.join(failed_files)}"
            else:
                status = "failure"
                message = "No documents were successfully inserted"
                if failed_files:
                    message += f". Failed files: {', '.join(failed_files)}"

            return InsertResponse(status=status, message=message)
        except Exception as e:
            await self._handle_exception(e, f"inserting batch from {directory_path}")
            raise

    async def scan_for_new_documents(self) -> Union[Any, HTTPValidationError]:
        """
        Start scanning LightRAG's /input directory for new documents.

        Returns:
            Union[Any, HTTPValidationError]: Operation result.
        """
        logger.debug("Starting scan for new documents...")
        return await self._call_api(
            api_func=async_scan_for_new_documents,
            operation_name="scanning for new documents",
            client=self.client,
        )

    async def get_documents(
        self,
    ) -> Union[DocsStatusesResponse, HTTPValidationError, None]:
        """
        Get list of all documents in LightRAG.

        Returns:
            Union[DocsStatusesResponse, HTTPValidationError]: List of documents.
        """
        logger.debug("Getting list of documents...")
        return await self._call_api(
            api_func=async_get_documents,
            operation_name="getting documents list",
            client=self.client,
        )

    async def get_pipeline_status(
        self,
    ) -> Union[PipelineStatusResponse, HTTPValidationError, None]:
        """
        Get status of document processing in pipeline.

        Returns:
            Union[PipelineStatusResponse, HTTPValidationError]: Pipeline status.
        """
        logger.debug("Getting pipeline status...")
        return await self._call_api(
            api_func=async_get_pipeline_status,
            operation_name="getting pipeline status",
            client=self.client,
        )

    async def get_graph_labels(self) -> Union[Dict[str, List[str]], HTTPValidationError, None]:
        """
        Get graph labels from knowledge graph.

        Returns:
            Union[Dict[str, List[str]], HTTPValidationError]: Graph labels.
        """
        logger.debug("Getting graph labels...")
        return await self._call_api(
            api_func=async_get_graph_labels,
            operation_name="getting graph labels",
            client=self.client,
        )

    async def delete_by_entity(
        self, entity_name: str
    ) -> Union[StatusMessageResponse, HTTPValidationError, None]:
        """
        Delete entity from knowledge graph by name.

        Args:
            entity_name (str): Entity name

        Returns:
            Union[StatusMessageResponse, HTTPValidationError]: Operation result.
        """
        logger.debug(f"Deleting entity by name: {entity_name}")

        return await self._call_api(
            api_func=async_delete_entity,
            operation_name=f"deletion by entity name: {entity_name}",
            client=self.client,
            entity_name=entity_name,
        )

    async def delete_by_doc_id(
        self, doc_id: str
    ) -> Union[StatusMessageResponse, HTTPValidationError, None]:
        """
        Delete all entities and relationships associated with a document.

        Args:
            doc_id (str): Document ID

        Returns:
            Union[StatusMessageResponse, HTTPValidationError]: Operation result.
        """
        logger.debug(f"Deleting entities by document ID: {doc_id}")

        return await self._call_api(
            api_func=async_delete_by_doc_id,
            operation_name=f"deletion by document ID: {doc_id}",
            client=self.client,
            doc_id=doc_id,
        )

    async def create_entity(
        self, entity_name: str, entity_type: str, description: str, source_id: str
    ) -> Union[EntityResponse, HTTPValidationError, None]:
        """
        Create new entity in knowledge graph.

        Args:
            entity_name (str): Entity name.
            entity_type (str): Entity type.
            description (str): Entity description.
            source_id (str): Source ID (document).

        Returns:
            Union[EntityResponse, HTTPValidationError]: Created entity.
        """
        logger.debug(f"Creating entity: {entity_name} (type={entity_type})")

        request = EntityRequest(
            entity_type=entity_type,
            description=description,
            source_id=source_id,
        )

        return await self._call_api(
            api_func=async_create_entity,
            operation_name="entity creation",
            client=self.client,
            entity_name=entity_name,
            body=request,
        )

    async def edit_entity(
        self, entity_name: str, entity_type: str, description: str, source_id: str
    ) -> Union[EntityResponse, HTTPValidationError, None]:
        """
        Edit existing entity in knowledge graph.

        Args:
            entity_name (str): Entity name.
            entity_type (str): New entity type.
            description (str): New entity description.
            source_id (str): Source ID (document).

        Returns:
            Union[EntityResponse, HTTPValidationError]: Updated entity.
        """
        logger.debug(f"Editing entity: {entity_name}")

        request = EntityRequest(
            entity_type=entity_type,
            description=description,
            source_id=source_id,
        )

        return await self._call_api(
            api_func=async_edit_entity,
            operation_name="entity editing",
            client=self.client,
            entity_name=entity_name,
            body=request,
        )

    async def create_relation(
        self,
        source: str,
        target: str,
        description: str,
        keywords: str,
        source_id: str | None,
        weight: float | None,
    ) -> Union[relation_response.RelationResponse, HTTPValidationError, None]:
        """
        Create relationship between entities in knowledge graph.

        Args:
            source (str): Source entity name.
            target (str): Target entity name.
            description (str): Relationship description.
            keywords (str): Keywords for relationship.
            source_id (str | None): Source ID (document).
            weight (float | None): Relationship weight.

        Returns:
            Union[relation_response.RelationResponse, HTTPValidationError]: Created relationship.
        """
        logger.debug(f"Creating relationship: {source} -> {target}")

        request = relation_request.RelationRequest(
            description=description,
            keywords=keywords,
            source_id=source_id,
            weight=weight,
        )

        return await self._call_api(
            api_func=async_create_relation,
            operation_name="relationship creation",
            client=self.client,
            source=source,
            target=target,
            body=request,
        )

    async def edit_relation(
        self,
        source: str,
        target: str,
        description: str,
        keywords: str,
        source_id: str | None,
        weight: float | None,
        relation_type: str,
    ) -> Union[relation_response.RelationResponse, HTTPValidationError, None]:
        """
        Edit relationship between entities.

        Args:
            source (str): Source entity name.
            target (str): Target entity name.
            properties (Dict[str, Any]): New relationship properties.

        Returns:
            Union[Dict, HTTPValidationError]: Updated relationship.
        """
        logger.debug(f"Editing relationship: {source} -> {target}")

        request = relation_request.RelationRequest(
            description=description,
            keywords=keywords,
            source_id=source_id,
            weight=weight,
        )

        return await self._call_api(
            api_func=async_edit_relation,
            operation_name="relationship editing",
            client=self.client,
            source=source,
            target=target,
            body=request,
            relation_type=relation_type,
        )

    async def merge_entities(
        self,
        source_entities: List[str],
        target_entity: str,
        merge_strategy: Dict[str, str],
    ) -> Union[EntityResponse, HTTPValidationError, None]:
        """
        Merge multiple entities into one with relationship migration.

        Args:
            source_entities (List[str]): List of entity names to merge.
            target_entity (str): Target entity name.
            merge_strategy (Dict[str, str], optional): Property merge strategy.
                Possible values for strategies: 'max', 'min', 'concat', 'first', 'last'
                Example: {"description": "concat", "weight": "max"}

        Returns:
            Union[Dict, HTTPValidationError]: Merge operation result.
        """
        logger.debug(f"Merging entities: {', '.join(source_entities)} -> {target_entity}")

        request = MergeEntitiesRequest(
            source_entities=source_entities,
            target_entity=target_entity,
            merge_strategy=MergeEntitiesRequestMergeStrategyType0.from_dict(merge_strategy),
        )

        return await self._call_api(
            api_func=async_merge_entities,
            operation_name="entity merging",
            client=self.client,
            body=request,
        )

    async def get_health(self) -> Union[Any, HTTPValidationError]:
        """
        Check health status of LightRAG service.

        Returns:
            Union[Any, HTTPValidationError]: Health status.
        """
        logger.debug("Checking service health status...")
        return await self._call_api(
            api_func=async_get_health,
            operation_name="health check",
            client=self.client,
        )

    async def close(self):
        """Close HTTP client."""
        await self.client.get_async_httpx_client().aclose()
        logger.info("LightRAG API client closed.")



"""Contains all the data models used in inputs/outputs"""

from .body_insert_batch_documents_file_batch_post import BodyInsertBatchDocumentsFileBatchPost
from .body_insert_file_documents_file_post import BodyInsertFileDocumentsFilePost
from .body_login_login_post import BodyLoginLoginPost
from .body_upload_to_input_dir_documents_upload_post import BodyUploadToInputDirDocumentsUploadPost
from .doc_status import DocStatus
from .doc_status_response import DocStatusResponse
from .doc_status_response_metadata_type_0 import DocStatusResponseMetadataType0
from .docs_statuses_response import DocsStatusesResponse
from .docs_statuses_response_statuses import DocsStatusesResponseStatuses
from .entity_request import EntityRequest
from .http_validation_error import HTTPValidationError
from .insert_response import InsertResponse
from .insert_text_request import InsertTextRequest
from .insert_texts_request import InsertTextsRequest
from .merge_entities_request import MergeEntitiesRequest
from .merge_entities_request_merge_strategy_type_0 import MergeEntitiesRequestMergeStrategyType0
from .ollama_chat_request import OllamaChatRequest
from .ollama_chat_request_options_type_0 import OllamaChatRequestOptionsType0
from .ollama_generate_request import OllamaGenerateRequest
from .ollama_generate_request_options_type_0 import OllamaGenerateRequestOptionsType0
from .ollama_message import OllamaMessage
from .pipeline_status_response import PipelineStatusResponse
from .pipeline_status_response_update_status_type_0 import PipelineStatusResponseUpdateStatusType0
from .query_request import QueryRequest
from .query_request_conversation_history_type_0_item import QueryRequestConversationHistoryType0Item
from .query_request_mode import QueryRequestMode
from .query_response import QueryResponse
from .relation_request import RelationRequest
from .validation_error import ValidationError

__all__ = (
    "BodyInsertBatchDocumentsFileBatchPost",
    "BodyInsertFileDocumentsFilePost",
    "BodyLoginLoginPost",
    "BodyUploadToInputDirDocumentsUploadPost",
    "DocsStatusesResponse",
    "DocsStatusesResponseStatuses",
    "DocStatus",
    "DocStatusResponse",
    "DocStatusResponseMetadataType0",
    "EntityRequest",
    "EntityRequestProperties",
    "HTTPValidationError",
    "InsertResponse",
    "InsertTextRequest",
    "InsertTextsRequest",
    "MergeEntitiesRequest",
    "MergeEntitiesRequestMergeStrategyType0",
    "OllamaChatRequest",
    "OllamaChatRequestOptionsType0",
    "OllamaGenerateRequest",
    "OllamaGenerateRequestOptionsType0",
    "OllamaMessage",
    "PipelineStatusResponse",
    "PipelineStatusResponseUpdateStatusType0",
    "QueryRequest",
    "QueryRequestConversationHistoryType0Item",
    "QueryRequestMode",
    "QueryResponse",
    "RelationRequest",
    "RelationRequestProperties",
    "ValidationError",
)



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.pipeline_status_response_update_status_type_0 import PipelineStatusResponseUpdateStatusType0


T = TypeVar("T", bound="PipelineStatusResponse")


@_attrs_define
class PipelineStatusResponse:
    """Response model for pipeline status

    Attributes:
        autoscanned: Whether auto-scan has started
        busy: Whether the pipeline is currently busy
        job_name: Current job name (e.g., indexing files/indexing texts)
        job_start: Job start time as ISO format string (optional)
        docs: Total number of documents to be indexed
        batchs: Number of batches for processing documents
        cur_batch: Current processing batch
        request_pending: Flag for pending request for processing
        latest_message: Latest message from pipeline processing
        history_messages: List of history messages
        update_status: Status of update flags for all namespaces

        Attributes:
            autoscanned (Union[Unset, bool]):  Default: False.
            busy (Union[Unset, bool]):  Default: False.
            job_name (Union[Unset, str]):  Default: 'Default Job'.
            job_start (Union[None, Unset, str]):
            docs (Union[Unset, int]):  Default: 0.
            batchs (Union[Unset, int]):  Default: 0.
            cur_batch (Union[Unset, int]):  Default: 0.
            request_pending (Union[Unset, bool]):  Default: False.
            latest_message (Union[Unset, str]):  Default: ''.
            history_messages (Union[None, Unset, list[str]]):
            update_status (Union['PipelineStatusResponseUpdateStatusType0', None, Unset]):
    """

    autoscanned: Union[Unset, bool] = False
    busy: Union[Unset, bool] = False
    job_name: Union[Unset, str] = "Default Job"
    job_start: Union[None, Unset, str] = UNSET
    docs: Union[Unset, int] = 0
    batchs: Union[Unset, int] = 0
    cur_batch: Union[Unset, int] = 0
    request_pending: Union[Unset, bool] = False
    latest_message: Union[Unset, str] = ""
    history_messages: Union[None, Unset, list[str]] = UNSET
    update_status: Union["PipelineStatusResponseUpdateStatusType0", None, Unset] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.pipeline_status_response_update_status_type_0 import PipelineStatusResponseUpdateStatusType0

        autoscanned = self.autoscanned

        busy = self.busy

        job_name = self.job_name

        job_start: Union[None, Unset, str]
        if isinstance(self.job_start, Unset):
            job_start = UNSET
        else:
            job_start = self.job_start

        docs = self.docs

        batchs = self.batchs

        cur_batch = self.cur_batch

        request_pending = self.request_pending

        latest_message = self.latest_message

        history_messages: Union[None, Unset, list[str]]
        if isinstance(self.history_messages, Unset):
            history_messages = UNSET
        elif isinstance(self.history_messages, list):
            history_messages = self.history_messages

        else:
            history_messages = self.history_messages

        update_status: Union[None, Unset, dict[str, Any]]
        if isinstance(self.update_status, Unset):
            update_status = UNSET
        elif isinstance(self.update_status, PipelineStatusResponseUpdateStatusType0):
            update_status = self.update_status.to_dict()
        else:
            update_status = self.update_status

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if autoscanned is not UNSET:
            field_dict["autoscanned"] = autoscanned
        if busy is not UNSET:
            field_dict["busy"] = busy
        if job_name is not UNSET:
            field_dict["job_name"] = job_name
        if job_start is not UNSET:
            field_dict["job_start"] = job_start
        if docs is not UNSET:
            field_dict["docs"] = docs
        if batchs is not UNSET:
            field_dict["batchs"] = batchs
        if cur_batch is not UNSET:
            field_dict["cur_batch"] = cur_batch
        if request_pending is not UNSET:
            field_dict["request_pending"] = request_pending
        if latest_message is not UNSET:
            field_dict["latest_message"] = latest_message
        if history_messages is not UNSET:
            field_dict["history_messages"] = history_messages
        if update_status is not UNSET:
            field_dict["update_status"] = update_status

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.pipeline_status_response_update_status_type_0 import PipelineStatusResponseUpdateStatusType0

        d = dict(src_dict)
        autoscanned = d.pop("autoscanned", UNSET)

        busy = d.pop("busy", UNSET)

        job_name = d.pop("job_name", UNSET)

        def _parse_job_start(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        job_start = _parse_job_start(d.pop("job_start", UNSET))

        docs = d.pop("docs", UNSET)

        batchs = d.pop("batchs", UNSET)

        cur_batch = d.pop("cur_batch", UNSET)

        request_pending = d.pop("request_pending", UNSET)

        latest_message = d.pop("latest_message", UNSET)

        def _parse_history_messages(data: object) -> Union[None, Unset, list[str]]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                history_messages_type_0 = cast(list[str], data)

                return history_messages_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, list[str]], data)

        history_messages = _parse_history_messages(d.pop("history_messages", UNSET))

        def _parse_update_status(data: object) -> Union["PipelineStatusResponseUpdateStatusType0", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                update_status_type_0 = PipelineStatusResponseUpdateStatusType0.from_dict(data)

                return update_status_type_0
            except:  # noqa: E722
                pass
            return cast(Union["PipelineStatusResponseUpdateStatusType0", None, Unset], data)

        update_status = _parse_update_status(d.pop("update_status", UNSET))

        pipeline_status_response = cls(
            autoscanned=autoscanned,
            busy=busy,
            job_name=job_name,
            job_start=job_start,
            docs=docs,
            batchs=batchs,
            cur_batch=cur_batch,
            request_pending=request_pending,
            latest_message=latest_message,
            history_messages=history_messages,
            update_status=update_status,
        )

        pipeline_status_response.additional_properties = d
        return pipeline_status_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
